--[[
	sptiCu   ver 0.02
	  Y.Kuze
	  2018.01.03
]]
require("alien")

SENSE_LENGTH = 32
BUF_LENGTH   = 512*1024

-- Mode Sense/Select page constants.
--

MODE_PAGE_ERROR_RECOVERY     =   0x01
MODE_PAGE_DISCONNECT         =   0x02
MODE_PAGE_FORMAT_DEVICE      =   0x03
MODE_PAGE_RIGID_GEOMETRY     =   0x04
MODE_PAGE_FLEXIBILE          =   0x05
MODE_PAGE_VERIFY_ERROR       =   0x07
MODE_PAGE_CACHING            =   0x08
MODE_PAGE_PERIPHERAL         =   0x09
MODE_PAGE_CONTROL            =   0x0A
MODE_PAGE_MEDIUM_TYPES       =   0x0B
MODE_PAGE_NOTCH_PARTITION    =   0x0C
MODE_SENSE_RETURN_ALL        =   0x3f
MODE_SENSE_CURRENT_VALUES    =   0x00
MODE_SENSE_CHANGEABLE_VALUES =   0x40
MODE_SENSE_DEFAULT_VAULES    =   0x80
MODE_SENSE_SAVED_VALUES      =   0xc0
MODE_PAGE_DEVICE_CONFIG      =   0x10
MODE_PAGE_MEDIUM_PARTITION   =   0x11
MODE_PAGE_DATA_COMPRESS      =   0x0f

--
-- SCSI CDB operation codes
--

SCSIOP_TEST_UNIT_READY    = 0x00
SCSIOP_REZERO_UNIT        = 0x01
SCSIOP_REWIND             = 0x01
SCSIOP_REQUEST_BLOCK_ADDR = 0x02
SCSIOP_REQUEST_SENSE      = 0x03
SCSIOP_FORMAT_UNIT        = 0x04
SCSIOP_READ_BLOCK_LIMITS  = 0x05
SCSIOP_REASSIGN_BLOCKS    = 0x07
SCSIOP_READ6              = 0x08
SCSIOP_RECEIVE            = 0x08
SCSIOP_WRITE6             = 0x0A
SCSIOP_PRINT              = 0x0A
SCSIOP_SEND               = 0x0A
SCSIOP_SEEK6              = 0x0B
SCSIOP_TRACK_SELECT       = 0x0B
SCSIOP_SLEW_PRINT         = 0x0B
SCSIOP_SEEK_BLOCK         = 0x0C
SCSIOP_PARTITION          = 0x0D
SCSIOP_READ_REVERSE       = 0x0F
SCSIOP_WRITE_FILEMARKS    = 0x10
SCSIOP_FLUSH_BUFFER       = 0x10
SCSIOP_SPACE              = 0x11
SCSIOP_INQUIRY            = 0x12
SCSIOP_VERIFY6            = 0x13
SCSIOP_RECOVER_BUF_DATA   = 0x14
SCSIOP_MODE_SELECT        = 0x15
SCSIOP_RESERVE_UNIT       = 0x16
SCSIOP_RELEASE_UNIT       = 0x17
SCSIOP_COPY               = 0x18
SCSIOP_ERASE              = 0x19
SCSIOP_MODE_SENSE         = 0x1A
SCSIOP_START_STOP_UNIT    = 0x1B
SCSIOP_STOP_PRINT         = 0x1B
SCSIOP_LOAD_UNLOAD        = 0x1B
SCSIOP_RECEIVE_DIAGNOSTIC = 0x1C
SCSIOP_SEND_DIAGNOSTIC    = 0x1D
SCSIOP_MEDIUM_REMOVAL     = 0x1E
SCSIOP_READ_CAPACITY      = 0x25
SCSIOP_READ               = 0x28
SCSIOP_WRITE              = 0x2A
SCSIOP_SEEK               = 0x2B
SCSIOP_LOCATE             = 0x2B
SCSIOP_WRITE_VERIFY       = 0x2E
SCSIOP_VERIFY             = 0x2F
SCSIOP_SEARCH_DATA_HIGH   = 0x30
SCSIOP_SEARCH_DATA_EQUAL  = 0x31
SCSIOP_SEARCH_DATA_LOW    = 0x32
SCSIOP_SET_LIMITS         = 0x33
SCSIOP_READ_POSITION      = 0x34
SCSIOP_SYNCHRONIZE_CACHE  = 0x35
SCSIOP_COMPARE            = 0x39
SCSIOP_COPY_COMPARE       = 0x3A
SCSIOP_WRITE_DATA_BUFF    = 0x3B
SCSIOP_READ_DATA_BUFF     = 0x3C
SCSIOP_CHANGE_DEFINITION  = 0x40
SCSIOP_READ_SUB_CHANNEL   = 0x42
SCSIOP_READ_TOC           = 0x43
SCSIOP_READ_HEADER        = 0x44
SCSIOP_PLAY_AUDIO         = 0x45
SCSIOP_PLAY_AUDIO_MSF     = 0x47
SCSIOP_PLAY_TRACK_INDEX   = 0x48
SCSIOP_PLAY_TRACK_RELATIVE= 0x49
SCSIOP_PAUSE_RESUME       = 0x4B
SCSIOP_LOG_SELECT         = 0x4C
SCSIOP_LOG_SENSE          = 0x4D

--
-- Define values for pass-through DataIn field.
--

SCSI_IOCTL_DATA_OUT         = 0
SCSI_IOCTL_DATA_IN          = 1
SCSI_IOCTL_DATA_UNSPECIFIED = 2

--=============================================================================
-- ]p[^`e[u (̊ɍ킹āAҏWč\܂B)
-------------------------------------------------------------------------------
-- ̃e[uɂ́A[XɈȉ̂̂܂݂܂B
--   Mt.FujiKiŒ`ꂽR}h
--   SCSI-2KiŒ`ꂽR}h(AAwǎgȂR}h()Ŋ)
--    - SűʃR}h
--    - Direct AccessũR}h
--    - CD-ROMũR}h
--    - Optical MemoryũR}h
--    - Write OnceũR}h
--    - Mecha ChangerũR}h
-------------------------------------------------------------------------------
--   Ⴆ΁AwXFER_DIRIN, 7, 2, 0, // 28h : READ(10)x̍śA
--   ]́A^[Qbg˃zXgŁA
--   ]́A  CDBByte7,8ł邱Ƃ܂B
-------------------------------------------------------------------------------
--   ̒`e[uŁȂR[hɉ삵ȂꍇA
--   wXFER_UNKNOWN, 0, 0 xƋLڂ悤ɂĉB
--=============================================================================

XFER_DIROUT = SCSI_IOCTL_DATA_OUT
XFER_DIRIN  = SCSI_IOCTL_DATA_IN
XFER_NODATA = SCSI_IOCTL_DATA_UNSPECIFIED
XFER_UNKNOWN= SCSI_IOCTL_DATA_UNSPECIFIED

XferDef = {
	{XFER_NODATA,			0, 0},	-- 00h : TEST UNIT READY
	{XFER_NODATA,			0, 0},	-- 01h : REZERO UNIT
	{XFER_UNKNOWN,			0, 0},	-- 02h :
	{XFER_DIRIN ,			4, 1},	-- 03h : REQUEST SENSE
	{XFER_DIROUT,	    	0,12},	-- 04h : FORMAT UNIT
	{XFER_UNKNOWN,			0, 0},	-- 05h :
	{XFER_UNKNOWN,			0, 0},	-- 06h :
	{XFER_UNKNOWN,			0, 0},	-- 07h : (REASSIGN BLOCK)
	{XFER_DIRIN ,	    	4, 1},	-- 08h : READ(6)
	{XFER_UNKNOWN,			0, 0},	-- 09h :
	{XFER_DIROUT,	    	4, 1},	-- 0Ah : WRITE(6)
	{XFER_NODATA,			0, 0},	-- 0Bh : SEEK(6)
	{XFER_UNKNOWN,			0, 0},	-- 0Ch :
	{XFER_UNKNOWN,			0, 0},	-- 0Dh :
	{XFER_UNKNOWN,			0, 0},	-- 0Eh :
	{XFER_UNKNOWN,			0, 0},	-- 0Fh :
	{XFER_UNKNOWN,			0, 0},	-- 10h :
	{XFER_UNKNOWN,			0, 0},	-- 11h :
	{XFER_DIRIN ,			4, 1},	-- 12h : INQUIRY
	{XFER_UNKNOWN,			0, 0},	-- 13h :
	{XFER_UNKNOWN,			0, 0},	-- 14h :
	{XFER_DIROUT,			4, 1},	-- 15h : MODE SELECT(6)
	{XFER_NODATA,			0, 0},	-- 16h : RESERVE
	{XFER_NODATA,			0, 0},	-- 17h : RELEASE
	{XFER_UNKNOWN,			0, 0},	-- 18h : (COPY)
	{XFER_UNKNOWN,			0, 0},	-- 19h :
	{XFER_DIRIN ,			4, 1},	-- 1Ah : MODE SENSE(6)
	{XFER_NODATA,			0, 0},	-- 1Bh : START/STOP UNIT
	{XFER_UNKNOWN,			0, 0},	-- 1Ch : (RECEIVE DIAGNOSTIC RESULT)
	{XFER_UNKNOWN,			0, 0},	-- 1Dh : (SEND DIAGNOSTIC)
	{XFER_NODATA,			0, 0},	-- 1Eh : PREVENT/ALLOW MEDIUM REMOVAL
	{XFER_UNKNOWN,			0, 0},	-- 1Fh :
	{XFER_UNKNOWN,			0, 0},	-- 20h :
	{XFER_UNKNOWN,			0, 0},	-- 21h :
	{XFER_UNKNOWN,			0, 0},	-- 22h :
	{XFER_DIRIN ,			7, 2},	-- 23h : READ FORMAT CAPACITY
	{XFER_UNKNOWN,			0, 0},	-- 24h :
	{XFER_DIRIN ,			0, 8},	-- 25h : READ CAPACITY
	{XFER_UNKNOWN,			0, 0},	-- 26h :
	{XFER_UNKNOWN,			0, 0},	-- 27h :
	{XFER_DIRIN ,			7, 2},	-- 28h : READ(10)
	{XFER_UNKNOWN,			0, 0},	-- 29h : (READ GENERATION for Optical)
	{XFER_DIROUT,			7, 2},	-- 2Ah : WRITE(10)
	{XFER_NODATA,			0, 0},	-- 2Bh : SEEK(10)
	{XFER_UNKNOWN,			0, 0},	-- 2Ch : (ERASE(10) for Optical)
	{XFER_UNKNOWN,			0, 0},	-- 2Dh : (READ UPDATE BLOCK(10) for Optical)
	{XFER_DIROUT,			7, 2},	-- 2Eh : WRITE AND VERIFY(10)
	{XFER_NODATA,			0, 0},	-- 2Fh : VERIFY(10)
	{XFER_UNKNOWN,			0, 0},	-- 30h : (SEARCH DATA (30h))
	{XFER_UNKNOWN,			0, 0},	-- 31h : (SEARCH DATA (31h))
	{XFER_UNKNOWN,			0, 0},	-- 32h : (SEARCH DATA (32h))
	{XFER_UNKNOWN,			0, 0},	-- 33h : (SET LIMIT)
	{XFER_UNKNOWN,			0, 0},	-- 34h : (PRE-FETCH)
	{XFER_NODATA,			0, 0},	-- 35h : SYNCHRONIZE CACHE
	{XFER_UNKNOWN,			0, 0},	-- 36h : (LOCK UNLOCK CACHE)
	{XFER_DIRIN ,			7, 2},	-- 37h : READ DEFECT DATA
	{XFER_UNKNOWN,			0, 0},	-- 38h : (MEDIUM SCAN for Optical)
	{XFER_UNKNOWN,			0, 0},	-- 39h : (COMPARE)
	{XFER_UNKNOWN,			0, 0},	-- 3Ah : (COPY AND VERIFY)
	{XFER_DIROUT,			6, 3},	-- 3Bh : WRITE BUFFER
	{XFER_DIRIN ,			6, 3},	-- 3Ch : READ BUFFER
	{XFER_UNKNOWN,			0, 0},	-- 3Dh : (UPDATE BLOCK for Optical)
	{XFER_DIRIN ,			6, 3},	-- 3Eh : READ LONG
	{XFER_DIROUT,			6, 3},	-- 3Fh : WRITE LONG
	{XFER_UNKNOWN,			0, 0},	-- 40h : (CHANGE DEFINITION)
	{XFER_UNKNOWN,			0, 0},	-- 41h : (WRITE SAME)
	{XFER_DIRIN ,			7, 2},	-- 42h : READ SUB-CHANNEL
	{XFER_DIRIN ,			7, 2},	-- 43h : READ TOC
	{XFER_DIRIN ,			7, 2},	-- 44h : READ HEADER
	{XFER_NODATA,			0, 0},	-- 45h : PLAY AUDIO(10)
	{XFER_DIRIN ,			7, 2},	-- 46h : GET CONFIGURATION
	{XFER_NODATA,			0, 0},	-- 47h : PLAY AUDIO MSF
	{XFER_NODATA,			0, 0},	-- 48h : PLAY AUDIO TRACK/INDEX
	{XFER_NODATA,			0, 0},	-- 49h : PLAY TRACK RELATIVE(10)
	{XFER_DIRIN ,			7, 2},	-- 4Ah : GET EVENT STATUS NOTIFICATION
	{XFER_NODATA,			0, 0},	-- 4Bh : PAUSE/RESUME
	{XFER_UNKNOWN,			0, 0},	-- 4Ch : (LOG SELECT)
	{XFER_UNKNOWN,			0, 0},	-- 4Dh : (LOG SENSE)
	{XFER_NODATA,			0, 0},	-- 4Eh : STOP PLAY/SCAN
	{XFER_UNKNOWN,			0, 0},	-- 4Fh :
	{XFER_UNKNOWN,			0, 0},	-- 50h :
	{XFER_DIRIN ,			7, 2},	-- 51h : READ DISC INFORMATION
	{XFER_DIRIN ,			7, 2},	-- 52h : READ TRACK/RZONE INFORMATION
	{XFER_NODATA,			0, 0},	-- 53h : RESERVE TRACK/RZONE
	{XFER_DIROUT,			7, 2},	-- 54h : SEND OPC INFORMATION
	{XFER_DIROUT,			7, 2},	-- 55h : MODE SELECT(10)
	{XFER_UNKNOWN,			0, 0},	-- 56h : (RESERVE(10))
	{XFER_UNKNOWN,			0, 0},	-- 57h : (RELEASE(10))
	{XFER_NODATA,			0, 0},	-- 58h : REPAIR RZONE
	{XFER_UNKNOWN,			0, 0},	-- 59h :
	{XFER_DIRIN ,			7, 2},	-- 5Ah : MODE SENSE(10)
	{XFER_NODATA,			0, 0},	-- 5Bh : CLOSE TRACK/RZONE/SESSION/BORDER
	{XFER_DIRIN ,			7, 2},	-- 5Ch : READ BUFFER CAPACITY
	{XFER_DIROUT,			6, 3},	-- 5Dh : SEND CUE SHEET
	{XFER_UNKNOWN,			0, 0},	-- 5Eh :
	{XFER_UNKNOWN,			0, 0},	-- 5Fh :
	{XFER_UNKNOWN,			0, 0},	-- 60h :
	{XFER_UNKNOWN,			0, 0},	-- 61h :
	{XFER_UNKNOWN,			0, 0},	-- 62h :
	{XFER_UNKNOWN,			0, 0},	-- 63h :
	{XFER_UNKNOWN,			0, 0},	-- 64h :
	{XFER_UNKNOWN,			0, 0},	-- 65h :
	{XFER_UNKNOWN,			0, 0},	-- 66h :
	{XFER_UNKNOWN,			0, 0},	-- 67h :
	{XFER_UNKNOWN,			0, 0},	-- 68h :
	{XFER_UNKNOWN,			0, 0},	-- 69h :
	{XFER_UNKNOWN,			0, 0},	-- 6Ah :
	{XFER_UNKNOWN,			0, 0},	-- 6Bh :
	{XFER_UNKNOWN,			0, 0},	-- 6Ch :
	{XFER_UNKNOWN,			0, 0},	-- 6Dh :
	{XFER_UNKNOWN,			0, 0},	-- 6Eh :
	{XFER_UNKNOWN,			0, 0},	-- 6Fh :
	{XFER_UNKNOWN,			0, 0},	-- 70h :
	{XFER_UNKNOWN,			0, 0},	-- 71h :
	{XFER_UNKNOWN,			0, 0},	-- 72h :
	{XFER_UNKNOWN,			0, 0},	-- 73h :
	{XFER_UNKNOWN,			0, 0},	-- 74h :
	{XFER_UNKNOWN,			0, 0},	-- 75h :
	{XFER_UNKNOWN,			0, 0},	-- 76h :
	{XFER_UNKNOWN,			0, 0},	-- 77h :
	{XFER_UNKNOWN,			0, 0},	-- 78h :
	{XFER_UNKNOWN,			0, 0},	-- 79h :
	{XFER_UNKNOWN,			0, 0},	-- 7Ah :
	{XFER_UNKNOWN,			0, 0},	-- 7Bh :
	{XFER_UNKNOWN,			0, 0},	-- 7Ch :
	{XFER_UNKNOWN,			0, 0},	-- 7Dh :
	{XFER_UNKNOWN,			0, 0},	-- 7Eh :
	{XFER_UNKNOWN,			0, 0},	-- 7Fh :
	{XFER_UNKNOWN,			0, 0},	-- 80h :
	{XFER_UNKNOWN,			0, 0},	-- 81h :
	{XFER_UNKNOWN,			0, 0},	-- 82h :
	{XFER_UNKNOWN,			0, 0},	-- 83h :
	{XFER_UNKNOWN,			0, 0},	-- 84h :
	{XFER_UNKNOWN,			0, 0},	-- 85h :
	{XFER_UNKNOWN,			0, 0},	-- 86h :
	{XFER_UNKNOWN,			0, 0},	-- 87h :
	{XFER_UNKNOWN,			0, 0},	-- 88h :
	{XFER_UNKNOWN,			0, 0},	-- 89h :
	{XFER_UNKNOWN,			0, 0},	-- 8Ah :
	{XFER_UNKNOWN,			0, 0},	-- 8Bh :
	{XFER_UNKNOWN,			0, 0},	-- 8Ch :
	{XFER_UNKNOWN,			0, 0},	-- 8Dh :
	{XFER_UNKNOWN,			0, 0},	-- 8Eh :
	{XFER_UNKNOWN,			0, 0},	-- 8Fh :
	{XFER_UNKNOWN,			0, 0},	-- 90h :
	{XFER_UNKNOWN,			0, 0},	-- 91h :
	{XFER_UNKNOWN,			0, 0},	-- 92h :
	{XFER_UNKNOWN,			0, 0},	-- 93h :
	{XFER_UNKNOWN,			0, 0},	-- 94h :
	{XFER_UNKNOWN,			0, 0},	-- 95h :
	{XFER_UNKNOWN,			0, 0},	-- 96h :
	{XFER_UNKNOWN,			0, 0},	-- 97h :
	{XFER_UNKNOWN,			0, 0},	-- 98h :
	{XFER_UNKNOWN,			0, 0},	-- 99h :
	{XFER_UNKNOWN,			0, 0},	-- 9Ah :
	{XFER_UNKNOWN,			0, 0},	-- 9Bh :
	{XFER_UNKNOWN,			0, 0},	-- 9Ch :
	{XFER_UNKNOWN,			0, 0},	-- 9Dh :
	{XFER_UNKNOWN,			0, 0},	-- 9Eh :
	{XFER_UNKNOWN,			0, 0},	-- 9Fh :
	{XFER_UNKNOWN,			0, 0},	-- A0h :
	{XFER_NODATA,			0, 0},	-- A1h : BLANK
	{XFER_DIROUT,			8, 2},	-- A2h : SEND EVENT
	{XFER_DIROUT,			8, 2},	-- A3h : SEND KEY
	{XFER_DIRIN ,			8, 2},	-- A4h : REPORT KEY
	{XFER_NODATA,			0, 0},	-- A5h : PLAY AUDIO(12)
	{XFER_UNKNOWN,			0, 0},	-- A6h : (EXCHANGE MEDIUM for Changer)
	{XFER_NODATA,			0, 0},	-- A7h : SET READ AHEAD
	{XFER_DIRIN ,			6, 4},	-- A8h : READ(12)
	{XFER_NODATA,			0, 0},	-- A9h : PLAY TRACK RELATIVE(12)
	{XFER_DIROUT,			6, 4},	-- AAh : WRITE(12)
	{XFER_UNKNOWN,			0, 0},	-- ABh :
	{XFER_DIRIN ,			0, 0},	-- ACh : GET PERFORMANCE [no limit length]
	{XFER_DIRIN ,			8, 2},	-- ADh : READ DVD STRUCTURE
	{XFER_DIROUT,			6, 4},	-- AEh : WRITE AND VERIFY(12)
	{XFER_NODATA,			0, 0},	-- AFh : VERIFY(12)
	{XFER_UNKNOWN,			0, 0},	-- B0h : (SEARCH DATA (12) (B0h) for Optical)
	{XFER_UNKNOWN,			0, 0},	-- B1h : (SEARCH DATA (12) (B1h) for Optical)
	{XFER_UNKNOWN,			0, 0},	-- B2h : (SEARCH DATA (12) (B2h) for Optical)
	{XFER_UNKNOWN,			0, 0},	-- B3h : (SET LIMIT (12) for Optical)
	{XFER_UNKNOWN,			0, 0},	-- B4h :
	{XFER_UNKNOWN,			0, 0},	-- B5h : (REQUEST VOLUME ELEMENT ADDRESS for Changer)
	{XFER_DIROUT,			9, 2},	-- B6h : SET STREAMING
	{XFER_UNKNOWN,			0, 0},	-- B7h : (READ DEFECT DATA(12) for Optical)
	{XFER_UNKNOWN,			0, 0},	-- B8h : (READ ELEMENT STATUS for Changer)
	{XFER_DIRIN ,			0, 0},	-- B9h : READ CD MSF [no limit length]
	{XFER_NODATA,			0, 0},	-- BAh : SCAN
	{XFER_NODATA,			0, 0},	-- BBh : SET CD SPEED
	{XFER_UNKNOWN,			0, 0},	-- BCh :
	{XFER_DIRIN ,			8, 2},	-- BDh : MECHANISM STATUS
	{XFER_DIRIN ,			6, 3},	-- BEh : READ CD
	{XFER_DIROUT,			8, 2},	-- BFh : SEND DVD STRUCTURE
	{XFER_UNKNOWN,			0, 0},	-- C0h :
	{XFER_UNKNOWN,			0, 0},	-- C1h :
	{XFER_UNKNOWN,			0, 0},	-- C2h :
	{XFER_UNKNOWN,			0, 0},	-- C3h :
	{XFER_UNKNOWN,			0, 0},	-- C4h :
	{XFER_UNKNOWN,			0, 0},	-- C5h :
	{XFER_UNKNOWN,			0, 0},	-- C6h :
	{XFER_UNKNOWN,			0, 0},	-- C7h :
	{XFER_UNKNOWN,			0, 0},	-- C8h :
	{XFER_UNKNOWN,			0, 0},	-- C9h :
	{XFER_UNKNOWN,			0, 0},	-- CAh :
	{XFER_UNKNOWN,			0, 0},	-- CBh :
	{XFER_UNKNOWN,			0, 0},	-- CCh :
	{XFER_UNKNOWN,			0, 0},	-- CDh :
	{XFER_UNKNOWN,			0, 0},	-- CEh :
	{XFER_UNKNOWN,			0, 0},	-- CFh :
	{XFER_UNKNOWN,			0, 0},	-- D0h :
	{XFER_UNKNOWN,			0, 0},	-- D1h :
	{XFER_UNKNOWN,			0, 0},	-- D2h :
	{XFER_UNKNOWN,			0, 0},	-- D3h :
	{XFER_UNKNOWN,			0, 0},	-- D4h :
	{XFER_UNKNOWN,			0, 0},	-- D5h :
	{XFER_UNKNOWN,			0, 0},	-- D6h :
	{XFER_UNKNOWN,			0, 0},	-- D7h :
	{XFER_UNKNOWN,			0, 0},	-- D8h :
	{XFER_UNKNOWN,			0, 0},	-- D9h :
	{XFER_UNKNOWN,			0, 0},	-- DAh :
	{XFER_UNKNOWN,			0, 0},	-- DBh :
	{XFER_UNKNOWN,			0, 0},	-- DCh :
	{XFER_UNKNOWN,			0, 0},	-- DDh :
	{XFER_UNKNOWN,			0, 0},	-- DEh :
	{XFER_UNKNOWN,			0, 0},	-- DFh :
	{XFER_UNKNOWN,			0, 0},	-- E0h :
	{XFER_UNKNOWN,			0, 0},	-- E1h :
	{XFER_UNKNOWN,			0, 0},	-- E2h :
	{XFER_UNKNOWN,			0, 0},	-- E3h :
	{XFER_UNKNOWN,			0, 0},	-- E4h :
	{XFER_UNKNOWN,			0, 0},	-- E5h :
	{XFER_UNKNOWN,			0, 0},	-- E6h :
	{XFER_UNKNOWN,			0, 0},	-- E7h :
	{XFER_UNKNOWN,			0, 0},	-- E8h :
	{XFER_UNKNOWN,			0, 0},	-- E9h :
	{XFER_UNKNOWN,			0, 0},	-- EAh :
	{XFER_UNKNOWN,			0, 0},	-- EBh :
	{XFER_UNKNOWN,			0, 0},	-- ECh :
	{XFER_UNKNOWN,			0, 0},	-- EDh :
	{XFER_UNKNOWN,			0, 0},	-- EEh :
	{XFER_UNKNOWN,			0, 0},	-- EFh :
	{XFER_UNKNOWN,			0, 0},	-- F0h :
	{XFER_UNKNOWN,			0, 0},	-- F1h :
	{XFER_UNKNOWN,			0, 0},	-- F2h :
	{XFER_UNKNOWN,			0, 0},	-- F3h :
	{XFER_UNKNOWN,			0, 0},	-- F4h :
	{XFER_UNKNOWN,			0, 0},	-- F5h :
	{XFER_UNKNOWN,			0, 0},	-- F6h :
	{XFER_UNKNOWN,			0, 0},	-- F7h :
	{XFER_UNKNOWN,			0, 0},	-- F8h :
	{XFER_UNKNOWN,			0, 0},	-- F9h :
	{XFER_UNKNOWN,			0, 0},	-- FAh :
	{XFER_UNKNOWN,			0, 0},	-- FBh :
	{XFER_UNKNOWN,			0, 0},	-- FCh :
	{XFER_UNKNOWN,			0, 0},	-- FDh :
	{XFER_UNKNOWN,			0, 0},	-- FEh :
	{XFER_UNKNOWN,			0, 0},	-- FFh :
};

--[[
	spti
	  ֐
	    exe             : SPTIR}hs
	    getHostErrMsg   : G[R[hG[bZ[Wɕϊ
	    getAlignmentMask: AlignmentMask擾(K؂alignmetMask擾)
	  
	  ϐ
	    portName      : PortNameB "D:", "E:" Ȃ
	    cdbBuf        : CDBmFpBݒɂ͎gpȂB
	    buf           : ^:R}hspBuffer (Read / WriteȂǂŎgp) ... Size: BUF_LENGTH
	    bufBuf        : R}hspBuffer (Read / WriteȂǂŎgp) ... Size: BUF_LENGTH
	    sense         : R}hsSenseCode.                        ... Size: SENSE_LENGTH
	    timeout       : R}h^CAEg. [sec] ftHg 10sec
	    errCode       : R}hsErrorCode. byteHost Status, ByteTarget Status
	    bytesRet      : R}hs bufԑ ByteTCY
	    alignmentMask : alignmentMaskmFpB
	  
	  ჌x֐
	    base
	    getHostError
	    getAlignmentMaskForDevice
]]
spti={ exe, base, getHostErrMsg, getHostError, getAlignmentMask, getAlignmentMaskForDevice, portName, cdbBuf, buf, bufBuf, sense, timeout, errCode, bytesRet, alignmentMask }
-- spti.exe ̐錾
-- int sptiexe(UCHAR *portname, UCHAR opt, UCHAR *cdb, UCHAR cdbsz, UCHAR *paraBuffer, ULONG dataSize, UCHAR *senseData, DWORD timeout, ULONG *errorCode, ULONG *returned) {
spti.base = alien.sptibase.sptiexe
spti.base:types{ ret = "int", abi = "stdcall", "string", "char", "string", "char", "pointer", "ulong", "pointer", "uint", "ref uint", "ref uint" }
-- void printError(UCHAR *errBuf, ULONG ErrorCode)
spti.getHostError = alien.sptibase.getHostError
spti.getHostError:types{ ret = "void", abi = "stdcall", "pointer", "ulong" }
-- UINT getAlignmentMaskForDevice(UCHAR *portname, PULONG AlignmentMask, ULONG *errorCode);
spti.getAlignmentMaskForDevice = alien.sptibase.getAlignmentMaskForDevice
spti.getAlignmentMaskForDevice:types{ ret = "uint", abi = "stdcall", "string", "ref uint", "ref uint" }

--[[
    
	  1: CDB        Table
	  2: BufSize    ȗ́AXferDefwɊÂ
	  3: Option     0:SCSI_IOCTL_DATA_OUT, 1:SCSI_IOCTL_DATA_IN, 2:SCSI_IOCTL_DATA_UNSPECIFIED
	                ȗ́AXferDefwɊÂ
	߂l
	  ErrorCode      byteHost Status, ByteTarget Status (spti.errCodeƓ)
	  BytesReturned  Bufɏo͂f[^TCY (spti.bytesRetƓ)
	֘A
	  spti.portName   "e:"
	  spti.cdbBuf
	  spti.buf  (spti.bufBuf)
	  spti.sense
	  spti.timeout   10 [sec]
	  spti.errCode
	  spti.bytesRet
	
	
	  spti.exe()
	  spti.exe({0,0,0,0,0,0}, BufSize)
	  spti.exe({0,0,0,0,0,0}, BufSize, SCSI_IOCTL_DATA_IN)
--]]
spti.exe = function(...)
	local CDB, BufSize, Option = ...
	local n = #{...}
	local cdbsz  = 0
	local cdbcmd = 0
	local ret    = 0
	
	if (n>=4)                      then  error("p[^sK؂ł", 2)      end
	if (n>=1) then
		if type(CDB)~="table"      then  error("p[^sK؂ł", 2)      end
		cdbsz = #CDB
		if cdbsz>16                then  error("CDBA16Byte𒴂Ă܂", 2) end
		for i=1, cdbsz do
			spti.cdbBuf[i] = CDB[i]
		end
		cdbcmd     = CDB[1]+1		-- tablel 1-baseȂ̂ +1
	end
	if n==0                        then
		BufSize = 0
		Option = XFER_NODATA
	end
	if (1<=n) and (n<3)            then  Option = XferDef[cdbcmd][1]             end
	if n==1                        then		-- XferDef莩BufSizeݒ
		local st   = XferDef[cdbcmd][2]+1
		local len  = XferDef[cdbcmd][3]
		
		if Option==XFER_NODATA then  BufSize = 0
		else
			BufSize = 0
			for i=st, st+len-1 do
				BufSize = BufSize * 0x100 + CDB[i]
			end
		end
	end
	if type(BufSize)~="number"     then  error("BufSizew error", 2)           end
	if type(Option)~="number"      then  error("Optionw error", 2)            end
	
	for i=1, BufSize do		-- Buffer̃Rs[
		spti.bufBuf[i] = string.byte(spti.buf,i)
	end
	spti.errCode  = 0
	spti.bytesRet = 0
	
	ret, spti.errCode, spti.bytesRet =  spti.base(spti.portName, Option, spti.cdbBuf, cdbsz, spti.bufBuf, BufSize, spti.sense, spti.timeout, spti.errCode, spti.bytesRet)
	if BufSize>=1 then spti.buf="" end
	for i=1, BufSize do		-- Buffer̃Rs[
			spti.buf = spti.buf..string.char(bit32.band(spti.bufBuf[i], 0xFF))
	end
	return spti.errCode, spti.bytesRet
end

--[[
    
	  ErrorCode
	֘A
	  spti.errCode
	
	  spti.getHostErrMsg()
	  spti.getHostErrMsg(errorCode)
--]]
spti.getHostErrMsg = function(ErrorCode)
	local errBuf = alien.buffer(80)
	
	if ErrorCode==nil then ErrorCode = spti.errCode end
	spti.getHostError(errBuf, ErrorCode)
	return errBuf:tostring()
end

--[[
    
	  Ȃ
	֘A
	  spti.portName
	  spti.alignmentMask
	  spti.errCode
	
	  spti.getAlignmentMask()
	  ret, alignmetMask = spti.getAlignmentMask()
]]
-- UINT getAlignmentMaskForDevice(UCHAR *portname, PULONG AlignmentMask, ULONG *errorCode);
spti.getAlignmentMask = function()
	local ret
	ret, spti.alignmentMask, spti.errCode = spti.getAlignmentMaskForDevice(spti.portName, spti.alignmentMask, spti.errCode)
	return ret, spti.alignmentMask, spti.errCode
end

--[[
    
	  buf: \Ώۂbuffer
	  st : Startʒu
	  ed : Endʒu
]]
function printBuf(buf, st, ed)
    local i
    local c
	
	for i=st,ed do
        c = buf[i]
		if c<0 then c=c+0x100 end
		printf("%02X ", c)
    end
end

-- 
spti.portName = "d:"
if spti.cdbBuf==nil then  spti.cdbBuf   = alien.buffer(16+1)  end
if spti.bufBuf==nil then  spti.bufBuf   = alien.buffer(BUF_LENGTH+1) end
if spti.sense==nil  then  spti.sense    = alien.buffer(SENSE_LENGTH+1) end
spti.buf      = string.rep(0, BUF_LENGTH)
spti.timeout  = 10
spti.errCode  = 0
spti.bytesRet = 0
spti.alignmentMask = 0
